local MAJOR, MINOR = "LibFail-1.0", "3"
local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end

local callbacks = LibStub("CallbackHandler-1.0"):New(lib)

lib.frame = CreateFrame("Frame")

local frame = lib.frame
frame:RegisterEvent("PLAYER_ENTERING_WORLD")

--[===[@debug@ 
local fail_events = {
	"Fail_Sartheron_LavaWaves",
	"Fail_Heigan_Dance",
	"Fail_Any_VoidZone",
	"Fail_Sapphiron_Breath",
	"Fail_Frogger",
	"Fail_Thaddius_PolaritySwitch",
	"Fail_Thaddius_Jump",
}


function lib:Test()
    local e = math.floor(math.random() * #fail_events) + 1
    local p = math.floor(math.random() * 5) + 1

    callbacks:Fire(fail_events[e], "Test"..p)
end

--@end-debug@]===]

function lib:GoInactive()
    if not self.active then return end

	self.LastTsunami = {}
	self.ChargeCounter = {}
	self.LastCharge = {}
	self.ThaddiusAlive = true;
	self.DeathTime = 0
	self.LastSlime = {}

    self.active = nil

    frame:RegisterEvent("RAID_ROSTER_UPDATE")
	frame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

    callbacks:Fire("Fail_Inactive")
end

lib.active = true
lib:GoInactive()

do
	local _, etype, f
	
	frame:SetScript("OnEvent", function (self, event, ...)
		if event == "COMBAT_LOG_EVENT_UNFILTERED" then
			_, etype = ...
			f = lib[etype]
	
			if f then
				f(lib, ...)
			end
			
			return
		end

		f = lib[event]

		if f then
			f(lib, ...)
		end
	end)
end


function lib:GoActive()
    if self.active then return end

    frame:UnregisterEvent("RAID_ROSTER_UPDATE")
	frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

	self.active = true

    callbacks:Fire("Fail_Active")
end


function lib:PLAYER_ENTERING_WORLD(...)
	if GetNumRaidMembers() > 0 then
        self:GoActive()
	else
        self:GoInactive()
	end
end

function lib:RAID_ROSTER_UPDATE(...)
    self:GoActive()
end

function lib:SPELL_DAMAGE(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool)
	if (sourceName=="Mirror Image") or (destName=="Mirror Image") then
		return
	end

	-- Lava Waves
	if (spellName=="Flame Tsunami") then
		if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
			if self.LastTsunami[destName] == nil then
				self.LastTsunami[destName] = timestamp
				
				callbacks:Fire("Fail_Sartheron_LavaWaves", destName)
			else
				if (timestamp - lib.LastTsunami[destName]) > 10 then
					callbacks:Fire("Fail_Sartheron_LavaWaves", destName)
				end
				self.LastTsunami[destName] = timestamp
			end
		end
	end

	-- Heigan Dance, Void Zones, Sapphiron's Breath, Frogger
	if (spellName=="Eruption") or (spellName=="Void Blast") or (spellName=="Frost Breath") or (spellName=="Explode") then
		if (sourceName:find("Fissure")) or (sourceName=="Sapphiron") or (sourceName=="Living Poison") then
			if (spellName=="Eruption") then
				self.FailType = "Heigan_Dance"
			elseif (spellName=="Void Blast") then
				self.FailType = "Any_VoidZone"
			elseif (spellName=="Frost Breath") then
				self.FailType = "Sapphiron_Breath"
			elseif (spellName=="Explode") then
				self.FailType = "Frogger"
			end
			if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				callbacks:Fire("Fail_"..self.FailType, destName)
			end
		end
	end

	-- Thaddius Polarity Switch
	if (self.ThaddiusAlive) then
		if (spellName=="Positive Charge") or (spellName=="Negative Charge") then
			if self.ChargeCounter[sourceName] == nil then
				self.ChargeCounter[sourceName] = 1
				self.LastCharge[sourceName] = timestamp
			elseif (timestamp - lib.LastCharge[sourceName]) < 2 then
				self.ChargeCounter[sourceName] = lib.ChargeCounter[sourceName] + 1
				self.LastCharge[sourceName] = timestamp
			else
				self.ChargeCounter[sourceName] = 1
				self.LastCharge[sourceName] = timestamp
			end

			if self.ChargeCounter[sourceName] == 3 then
				callbacks:Fire("Fail_Thaddius_PolaritySwitch", sourceName)
			end
		end
	end
end

function lib:SPELL_CAST_START(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool)
	if sourceName=="Thaddius" then
		if spellName=="Polarity Shift" then
			wipe(self.ChargeCounter)
			self.ThaddiusAlive = true;
		end
	end
end

function lib:UNIT_DIED(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags)
	if (destName=="Thaddius") then
		self.ThaddiusAlive = false;
	elseif (destName=="Stalagg") or (destName=="Feugen") then
		self.DeathTime = timestamp
	end
end

function lib:SPELL_AURA_APPLIED(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool)
	if (spellName=="Slime") then
		if (timestamp - self.DeathTime) < 120 then -- error
			if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				if self.LastSlime[destName] == nil then
					callbacks:Fire("Fail_Thaddius_Jump", destName)
				elseif (timestamp - self.LastSlime[destName]) > 5 then
					callbacks:Fire("Fail_Thaddius_Jump", destName)
				end
			end
		end
	end
end

function lib:SPELL_AURA_REMOVED(timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellId, spellName, spellSchool)
	if (spellName=="Slime") then
		if (timestamp - self.DeathTime) < 120 then --error
			if bit.band(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 then
				self.LastSlime[destName] = timestamp
			end
		end
	end
end



